Notes
-----

Generic notes and tips:

* Alembic migrations may be tricky to be branched and merged.
  Consider postponing writing migrations until feature is ready to be
  merged.  Keep TODOs in your code for each migration that needs to be
  added.

* By default, Alembic reads configuration from system locations like
  `/usr/share` and `/etc`.  This behaviour can be overridden by
  setting `KOSCHEI_CONFIG` environment variable.

* In cases of weird tracebacks, make sure that there are no suprious
  uncommited files in `alembic/versions/`.

* Autogenerated revisions may not work as expected.  It may be safer
  and easier to write them manually.  In cases of new additions to DB
  model it may be possible to run tests, then just copy relevant DDL
  lines from schema dump, i.e. output of `pg_dump -s koschei_testdb`
  and finally manually add corresponding `DROP` statements.


Recommended workflow
--------------------

  1. Develop code change first, without any Alembic revisions.
     Document any model changes with "TODO migrations" as you make
     them.

  2. Before merging code to master, review changes to model.  Look for
     changes that require migrations, especially TODO items added in
     step 1.  Group changes in logically-consistent units.

  3. For each logical model change identified in step 2, write on a
     side SQL code for upgrades and downgrades.  DDL can be copied
     from schema dump of test DB.

  4. For each logical model change, add Alembic revision.  Generate
     new revision, add it to gid, fill template with upgrade and
     downgrade code prepared in step 3, commit to git.  Test by saving
     schema dump, downgrading DB, upgrading DB, and then comparing
     schema with initial one.

  5. Test non-trivial migrations on production DB dump.


Cheat-sheet
-----------

To set environment variables and aliases:

    . aux/set-env.sh
    alias koschei_alembic="KOSCHEI_CONFIG=$PWD/config.cfg.template:<(echo 'config={\"database_config\":{\"database\":\"koschei\"}}') alembic -c $PWD/alembic.ini"

To create clean DB for tests:

    rm -rf test/db/
    pg_init
    pg_start

To check current revision in filesystem:

    koschei_alembic show head

To check current revision in database:

    psql koschei <<<'SELECT * FROM alembic_version'

To upgrade database to latest revision from filesystem:

    koschei_alembic upgrade head

Run tests and dump DB schema:

    nosetests -v
    pg_dump -s koschei_testdb | less

Add new Alembic revision, to manually enter create and drop DDL
instructions:

    koschei_alembic revision -m 'Alter table foo, add column bar'
    git add alembic/versions/

To test upgrade or downgrade of newly-created revision:

    koschei_alembic downgrade -1
    koschei_alembic upgrade +1
